#ifndef VCA_CUSTLOG_H
#define VCA_CUSTLOG_H

#include <stdlib.h>
#include <stdarg.h>
#include <errno.h>
#include <string.h>

#include "can_vca.h"

//====================== logging support =======================
// -------------- log levels ------------------
#define LOG_FATAL   0
#define LOG_ERR     1
#define LOG_MSG     2
#define LOG_INF     3
#define LOG_DEB     4
#define LOG_TRASH   5

#define VCA_DECLARE_CUSTOM_LOG(vcapollfdnodecan) \
static void vlog(int level, const char *format, va_list ap) VCA_GNUC_UNUSED; \
static void vlog(int level, const char *format, va_list ap) \
{ \
    vca_vlog(#vcapollfdnodecan, level, format, ap); \
} \
 \
static void clog(int level, const char *format, ...) VCA_GNUC_UNUSED; \
static void clog(int level, const char *format, ...) \
{ \
    va_list ap; \
    va_start(ap, format); \
    vlog(level, format, ap); \
    va_end(ap); \
} \
 \
static void logerr(const char *format, ...) VCA_GNUC_UNUSED; \
static void logerr(const char *format, ...) \
{ \
    va_list ap; \
    va_start(ap, format); \
    vlog(LOG_ERR, format, ap); \
    va_end(ap); \
    clog(LOG_ERR, "\t%s\n", strerror(errno)); \
} \
 \
static void fatal_error(const char *format, ...) VCA_GNUC_UNUSED; \
static void fatal_error(const char *format, ...) \
{ \
    va_list ap; \
    va_start(ap, format); \
    vlog(LOG_FATAL, format, ap); \
    va_end(ap); \
    clog(LOG_ERR, "\t%s\n", strerror(errno)); \
    exit(EXIT_FAILURE); \
}
 
//=============================================================

#endif // VCA_CUSTLOG_H

